
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#
#

include logging
logdebug "Load tempdir package"


include ifs
include stacks
include atexit

newstack __LECABASHLIB_BASETMPDIRSTACK__
newstack __LECABASHLIB_TMPDIRSTACK__

__LECABASHLIB_TMPBASENAME__="__tmp__$(whoami).${$}__lecabashlib__.XXXXX"

function __remove_tmp__() {
	loginfo "Cleaning temporary directories"
	
	while popvalue __LECABASHLIB_TMPDIRSTACK__ directory ; do		
		loginfo "Removing tempdir : $directory" 
		rm -rf "$directory"
	done
}

registeratexit __remove_tmp__


function availabledisk() {
  	local FILENAME="${1}"


	pushifs $'\n'
  
	local lines=( $(df -k -P "${FILENAME}") )
	
	pushifs ' '
	local fields=(${lines[0]})
	local values=(${lines[1]})
	popifs
	popifs
	
	nfield=${#fields[@]}
	i=0
	while (( i < nfield )) && [[  "$(lower ${fields[$i]})" != "available" ]]  ; do
	 i=$(( i + 1 ))
	done
	
	
	if (( i < nfield )) ; then
		size=$(( values[i] / 1024 / 1024 ))
		echo $size
		return 0
	else
		return 1
	fi

}

function registertmpdir() {
	pushvalue __LECABASHLIB_BASETMPDIRSTACK__ "${1}"
}

function getbasetmpdir() {
	echo "${__LECABASHLIB_BASETMPDIRSTACK__[*]}"
}

function tempdirectory() {
	local __tempdirectory_minsize="0"

    if [[ ! -z "${1}" ]]; then
      local __tempdirectory_COMMAND="${1}="
    else
      local __tempdirectory_COMMAND="echo "
    fi
	shift
	
	if [[ "$(stacksize __LECABASHLIB_BASETMPDIRSTACK__)" == "0" ]]; then
		if [[ ! -z "$TMPDIR" ]] ; then
			registertmpdir "$TMPDIR"
		else
			mkdir -p "$HOME/tmp"
			registertmpdir "$HOME/tmp"
		fi
	fi

	if [[ ! -z "${1}" ]] ; then
		__tempdirectory_minsize="${1}"
	fi
	
	local __tempdirectory_nbase=$(stacksize __LECABASHLIB_BASETMPDIRSTACK__)
	local __tempdirectory_i=0
	
	while (( __tempdirectory_i < __tempdirectory_nbase )) && \
	      (( $(availabledisk "${__LECABASHLIB_BASETMPDIRSTACK__[$i]}") < __tempdirectory_minsize )) ; do
		__tempdirectory_i=$(( __tempdirectory_i + 1 ))
	done
	
	if (( __tempdirectory_i >= __tempdirectory_nbase )) ; then
		return 1
	fi

	local __tempdirectory_basedir="${__LECABASHLIB_BASETMPDIRSTACK__[$__tempdirectory_i]}"
	local __tempdirectory_directory=$(mktemp -d "${__tempdirectory_basedir%/}/${__LECABASHLIB_TMPBASENAME__}") 
	pushvalue __LECABASHLIB_TMPDIRSTACK__ "$__tempdirectory_directory" 
	
    local __tempdirectory_valcmd="${__tempdirectory_COMMAND}${__tempdirectory_directory}"
    eval $__tempdirectory_valcmd
	
}

